"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _Client_encryptor, _Client_transport, _Client_connection, _Client_gameAddr, _Client_addr, _Client_opHist, _Client_secretState;
Object.defineProperty(exports, "__esModule", { value: true });
exports.Client = void 0;
const connection_1 = require("./connection");
const secret_state_1 = require("./secret-state");
const events_1 = require("./events");
class Client {
    constructor(addr, gameAddr, transport, encryptor, connection) {
        _Client_encryptor.set(this, void 0);
        _Client_transport.set(this, void 0);
        _Client_connection.set(this, void 0);
        _Client_gameAddr.set(this, void 0);
        _Client_addr.set(this, void 0);
        _Client_opHist.set(this, void 0);
        _Client_secretState.set(this, void 0);
        __classPrivateFieldSet(this, _Client_addr, addr, "f");
        __classPrivateFieldSet(this, _Client_gameAddr, gameAddr, "f");
        __classPrivateFieldSet(this, _Client_transport, transport, "f");
        __classPrivateFieldSet(this, _Client_encryptor, encryptor, "f");
        __classPrivateFieldSet(this, _Client_connection, connection, "f");
        __classPrivateFieldSet(this, _Client_opHist, new Array(), "f");
        __classPrivateFieldSet(this, _Client_secretState, new secret_state_1.SecretState(encryptor), "f");
    }
    async attachGame() {
        const key = await __classPrivateFieldGet(this, _Client_encryptor, "f").exportPublicKey(undefined);
        await __classPrivateFieldGet(this, _Client_connection, "f").attachGame(__classPrivateFieldGet(this, _Client_gameAddr, "f"), new connection_1.AttachGameParams({
            signer: __classPrivateFieldGet(this, _Client_addr, "f"),
            key,
        }));
    }
    async submitEvent(event) {
        await __classPrivateFieldGet(this, _Client_connection, "f").submitEvent(__classPrivateFieldGet(this, _Client_gameAddr, "f"), new connection_1.SubmitEventParams({
            event,
        }));
    }
    async submitCustomEvent(customEvent) {
        const event = (0, events_1.makeCustomEvent)(__classPrivateFieldGet(this, _Client_gameAddr, "f"), customEvent);
        await __classPrivateFieldGet(this, _Client_connection, "f").submitEvent(__classPrivateFieldGet(this, _Client_gameAddr, "f"), new connection_1.SubmitEventParams({
            event,
        }));
    }
    async handleDecision(ctx) {
        return [];
    }
    loadRandomStates(ctx) {
        for (let randomState of ctx.randomStates) {
            if (!__classPrivateFieldGet(this, _Client_secretState, "f").isRandomLoaded(randomState.id)) {
                __classPrivateFieldGet(this, _Client_secretState, "f").genRandomStates(randomState.id, randomState.size);
            }
        }
    }
    async handleUpdatedContext(ctx) {
        this.loadRandomStates(ctx);
        const events = await this.handleDecision(ctx);
        return events;
    }
    flushSecretStates() {
        __classPrivateFieldGet(this, _Client_secretState, "f").clear();
        __classPrivateFieldGet(this, _Client_opHist, "f").splice(0);
    }
    async decrypt(ctx, randomId) {
        let randomState = ctx.getRandomState(randomId);
        let options = randomState.options;
        let revealed = await __classPrivateFieldGet(this, _Client_encryptor, "f").decryptWithSecrets(randomState.listRevealedCiphertexts(), randomState.listRevealedSecrets(), options);
        let assigned = await __classPrivateFieldGet(this, _Client_encryptor, "f").decryptWithSecrets(randomState.listAssignedCiphertexts(__classPrivateFieldGet(this, _Client_addr, "f")), randomState.listSharedSecrets(__classPrivateFieldGet(this, _Client_addr, "f")), options);
        return new Map([...revealed, ...assigned]);
    }
}
exports.Client = Client;
_Client_encryptor = new WeakMap(), _Client_transport = new WeakMap(), _Client_connection = new WeakMap(), _Client_gameAddr = new WeakMap(), _Client_addr = new WeakMap(), _Client_opHist = new WeakMap(), _Client_secretState = new WeakMap();
//# sourceMappingURL=client.js.map