import { GameAccount, GameBundle, PlayerJoin, ServerJoin } from './accounts';
import { GameEvent } from './events';
import { GameContext } from './game-context';
import { IEncryptor } from './encryptor';
import { Client } from './client';
import { DecryptionCache } from './decryption-cache';
/**
 * A subset of GameAccount, used in handler initialization.
 */
export interface IInitAccount {
    addr: string;
    players: PlayerJoin[];
    servers: ServerJoin[];
    data: Uint8Array;
    accessVersion: bigint;
    settleVersion: bigint;
    maxPlayers: number;
    minDeposit: bigint;
    maxDeposit: bigint;
}
export declare class InitAccount {
    readonly addr: string;
    readonly players: PlayerJoin[];
    readonly servers: ServerJoin[];
    readonly data: Uint8Array;
    readonly accessVersion: bigint;
    readonly settleVersion: bigint;
    readonly maxPlayers: number;
    readonly minDeposit: bigint;
    readonly maxDeposit: bigint;
    constructor(fields: IInitAccount);
    static createFromGameAccount(gameAccount: GameAccount, transactorAccessVersion: bigint, transactorSettleVersion: bigint): InitAccount;
    serialize(): Uint8Array;
    static deserialize(data: Uint8Array): InitAccount;
}
export interface IHandler {
    handleEvent(context: GameContext, event: GameEvent): Promise<void>;
    initState(context: GameContext, initAccount: InitAccount): Promise<void>;
}
export declare class Handler implements IHandler {
    #private;
    constructor(instance: WebAssembly.Instance, encryptor: IEncryptor, client: Client, decryptionCache: DecryptionCache);
    static initialize(gameBundle: GameBundle, encryptor: IEncryptor, client: Client, decryptionCache: DecryptionCache): Promise<Handler>;
    handleEvent(context: GameContext, event: GameEvent): Promise<void>;
    initState(context: GameContext, initAccount: InitAccount): Promise<void>;
    generalPreInitState(_context: GameContext, _initAccount: InitAccount): Promise<void>;
    generalPostInitState(_context: GameContext, _initAccount: InitAccount): Promise<void>;
    generalPreHandleEvent(context: GameContext, event: GameEvent, encryptor: IEncryptor): Promise<void>;
    generalPostHandleEvent(context: GameContext, event: GameEvent): Promise<void>;
    customInitState(context: GameContext, initAccount: InitAccount): Promise<void>;
    customHandleEvent(context: GameContext, event: GameEvent): Promise<void>;
}
