var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _AppClient_gameAddr, _AppClient_handler, _AppClient_wallet, _AppClient_client, _AppClient_transport, _AppClient_connection, _AppClient_gameContext, _AppClient_initGameAccount, _AppClient_onEvent, _AppClient_onMessage, _AppClient_encryptor, _AppClient_profileCaches, _AppClient_info, _AppClient_decryptionCache;
import { BroadcastFrameEvent, BroadcastFrameInit, BroadcastFrameMessage, Connection, SubmitEventParams, SubmitMessageParams, SubscribeEventParams, } from './connection';
import { GameContext } from './game-context';
import { GameContextSnapshot } from './game-context-snapshot';
import { Handler, InitAccount } from './handler';
import { Encryptor } from './encryptor';
import { SdkError } from './error';
import { Client } from './client';
import { Custom } from './events';
import { ProfileCache } from './profile-cache';
import { DecryptionCache } from './decryption-cache';
export class AppClient {
    constructor(gameAddr, handler, wallet, client, transport, connection, gameContext, initGameAccount, onEvent, onMessage, encryptor, info, decryptionCache) {
        _AppClient_gameAddr.set(this, void 0);
        _AppClient_handler.set(this, void 0);
        _AppClient_wallet.set(this, void 0);
        _AppClient_client.set(this, void 0);
        _AppClient_transport.set(this, void 0);
        _AppClient_connection.set(this, void 0);
        _AppClient_gameContext.set(this, void 0);
        _AppClient_initGameAccount.set(this, void 0);
        _AppClient_onEvent.set(this, void 0);
        _AppClient_onMessage.set(this, void 0);
        _AppClient_encryptor.set(this, void 0);
        _AppClient_profileCaches.set(this, void 0);
        _AppClient_info.set(this, void 0);
        _AppClient_decryptionCache.set(this, void 0);
        __classPrivateFieldSet(this, _AppClient_gameAddr, gameAddr, "f");
        __classPrivateFieldSet(this, _AppClient_handler, handler, "f");
        __classPrivateFieldSet(this, _AppClient_wallet, wallet, "f");
        __classPrivateFieldSet(this, _AppClient_client, client, "f");
        __classPrivateFieldSet(this, _AppClient_transport, transport, "f");
        __classPrivateFieldSet(this, _AppClient_connection, connection, "f");
        __classPrivateFieldSet(this, _AppClient_gameContext, gameContext, "f");
        __classPrivateFieldSet(this, _AppClient_initGameAccount, initGameAccount, "f");
        __classPrivateFieldSet(this, _AppClient_onEvent, onEvent, "f");
        __classPrivateFieldSet(this, _AppClient_onMessage, onMessage, "f");
        __classPrivateFieldSet(this, _AppClient_encryptor, encryptor, "f");
        __classPrivateFieldSet(this, _AppClient_profileCaches, new ProfileCache(transport), "f");
        __classPrivateFieldSet(this, _AppClient_info, info, "f");
        __classPrivateFieldSet(this, _AppClient_decryptionCache, decryptionCache, "f");
    }
    static async initialize(opts) {
        const { transport, wallet, gameAddr, onEvent, onMessage, storage } = opts;
        console.group('AppClient initialization');
        try {
            const playerAddr = wallet.walletAddr;
            const encryptor = await Encryptor.create(playerAddr, storage);
            const gameAccount = await transport.getGameAccount(gameAddr);
            if (gameAccount === undefined) {
                throw SdkError.gameAccountNotFound(gameAddr);
            }
            console.log('Game account:', gameAccount);
            const gameBundle = await transport.getGameBundle(gameAccount.bundleAddr);
            if (gameBundle === undefined) {
                throw SdkError.gameBundleNotFound(gameAccount.bundleAddr);
            }
            console.log('Game bundle:', gameBundle);
            const transactorAddr = gameAccount.transactorAddr;
            if (transactorAddr === undefined) {
                throw SdkError.gameNotServed(gameAddr);
            }
            console.log('Transactor address:', transactorAddr);
            const transactorAccount = await transport.getServerAccount(transactorAddr);
            if (transactorAccount === undefined) {
                throw SdkError.transactorAccountNotFound(transactorAddr);
            }
            const decryptionCache = new DecryptionCache();
            const connection = Connection.initialize(playerAddr, transactorAccount.endpoint, encryptor);
            const client = new Client(playerAddr, gameAddr, transport, encryptor, connection);
            const handler = await Handler.initialize(gameBundle, encryptor, client, decryptionCache);
            const gameContext = new GameContext(gameAccount);
            const token = await transport.getToken(gameAccount.tokenAddr);
            if (token === undefined) {
                throw SdkError.tokenNotFound(gameAccount.tokenAddr);
            }
            const info = {
                title: gameAccount.title,
                minDeposit: gameAccount.minDeposit,
                maxDeposit: gameAccount.maxDeposit,
                maxPlayers: gameAccount.maxPlayers,
                token,
            };
            return new AppClient(gameAddr, handler, wallet, client, transport, connection, gameContext, gameAccount, onEvent, onMessage, encryptor, info, decryptionCache);
        }
        finally {
            console.groupEnd();
        }
    }
    get playerAddr() {
        return __classPrivateFieldGet(this, _AppClient_wallet, "f").walletAddr;
    }
    get gameAddr() {
        return __classPrivateFieldGet(this, _AppClient_gameAddr, "f");
    }
    get gameContext() {
        return __classPrivateFieldGet(this, _AppClient_gameContext, "f");
    }
    /**
     * Get player profile by its wallet address.
     */
    async getProfile(addr) {
        return await __classPrivateFieldGet(this, _AppClient_transport, "f").getPlayerProfile(addr);
    }
    async invokeEventCallback(event) {
        const snapshot = new GameContextSnapshot(__classPrivateFieldGet(this, _AppClient_gameContext, "f"));
        await __classPrivateFieldGet(this, _AppClient_profileCaches, "f").injectProfiles(snapshot);
        const state = __classPrivateFieldGet(this, _AppClient_gameContext, "f").handlerState;
        __classPrivateFieldGet(this, _AppClient_onEvent, "f").call(this, snapshot, state, event);
    }
    /**
     * Connect to the transactor and retrieve the event stream.
     */
    async attachGame() {
        await __classPrivateFieldGet(this, _AppClient_client, "f").attachGame();
        const settleVersion = __classPrivateFieldGet(this, _AppClient_gameContext, "f").settleVersion;
        let sub = __classPrivateFieldGet(this, _AppClient_connection, "f").subscribeEvents(__classPrivateFieldGet(this, _AppClient_gameAddr, "f"), new SubscribeEventParams({ settleVersion }));
        for await (const frame of sub) {
            if (frame instanceof BroadcastFrameInit) {
                console.group('Initialize handler state');
                try {
                    const { checkpointState, accessVersion, settleVersion } = frame;
                    console.log('Access version:', accessVersion);
                    console.log('Settle version:', settleVersion);
                    __classPrivateFieldGet(this, _AppClient_gameContext, "f").applyCheckpoint(accessVersion, settleVersion);
                    const initAccount = InitAccount.createFromGameAccount(__classPrivateFieldGet(this, _AppClient_initGameAccount, "f"), accessVersion, settleVersion);
                    console.log('Init account:', initAccount);
                    await __classPrivateFieldGet(this, _AppClient_handler, "f").initState(__classPrivateFieldGet(this, _AppClient_gameContext, "f"), initAccount);
                    __classPrivateFieldGet(this, _AppClient_gameContext, "f").handlerState = checkpointState;
                    console.log('Context created:', __classPrivateFieldGet(this, _AppClient_gameContext, "f"));
                    await this.invokeEventCallback(undefined);
                }
                finally {
                    console.groupEnd();
                }
            }
            else if (frame instanceof BroadcastFrameMessage) {
                const { message } = frame;
                __classPrivateFieldGet(this, _AppClient_onMessage, "f").call(this, message);
            }
            else if (frame instanceof BroadcastFrameEvent) {
                const t0 = new Date().getTime();
                const { event, timestamp } = frame;
                console.group('Handle event: ' + event.kind());
                try {
                    __classPrivateFieldGet(this, _AppClient_gameContext, "f").prepareForNextEvent(timestamp);
                    try {
                        let context = new GameContext(__classPrivateFieldGet(this, _AppClient_gameContext, "f"));
                        await __classPrivateFieldGet(this, _AppClient_handler, "f").handleEvent(context, event);
                        __classPrivateFieldSet(this, _AppClient_gameContext, context, "f");
                    }
                    catch (err) {
                        console.error(err);
                    }
                    console.log("Cost: ", new Date().getTime() - t0, "ms");
                    await this.invokeEventCallback(event);
                }
                catch (e) {
                    console.log("Game context in error:", __classPrivateFieldGet(this, _AppClient_gameContext, "f"));
                    throw e;
                }
                finally {
                    console.groupEnd();
                }
            }
            else {
                break;
            }
        }
    }
    /**
     * Join game.
     */
    async join(params) {
        const gameAccount = await __classPrivateFieldGet(this, _AppClient_transport, "f").getGameAccount(this.gameAddr);
        if (gameAccount === undefined) {
            throw new Error('Game account not found');
        }
        const playersCount = gameAccount.players.length;
        if (gameAccount.maxPlayers <= playersCount) {
            throw new Error('Game is full');
        }
        let position = params.position;
        if (position === undefined) {
            for (let i = 0; i < gameAccount.maxPlayers; i++) {
                if (gameAccount.players.find(p => p.position === i) === undefined) {
                    position = i;
                    break;
                }
            }
        }
        if (position === undefined) {
            throw new Error('Game is full');
        }
        const publicKey = await __classPrivateFieldGet(this, _AppClient_encryptor, "f").exportPublicKey();
        __classPrivateFieldGet(this, _AppClient_transport, "f").join(__classPrivateFieldGet(this, _AppClient_wallet, "f"), {
            gameAddr: this.gameAddr,
            amount: params.amount,
            accessVersion: gameAccount.accessVersion,
            position,
            verifyKey: publicKey.ec,
        });
    }
    async submitEvent(arg) {
        let raw = arg instanceof Uint8Array ? arg : arg.serialize();
        const event = new Custom({ sender: this.playerAddr, raw });
        await __classPrivateFieldGet(this, _AppClient_connection, "f").submitEvent(__classPrivateFieldGet(this, _AppClient_gameAddr, "f"), new SubmitEventParams({
            event,
        }));
    }
    /**
     * Submit a message, contains arbitrary content.
     */
    async submitMessage(message) {
        await __classPrivateFieldGet(this, _AppClient_connection, "f").submitMessage(__classPrivateFieldGet(this, _AppClient_gameAddr, "f"), new SubmitMessageParams({
            content: message,
        }));
    }
    /**
     * Get hidden knowledge by random id. The result contains both
     * public and private information.  For performance reason, it's
     * better to cache the result somewhere instead of calling this
     * function frequently.
     */
    async getRevealed(randomId) {
        return __classPrivateFieldGet(this, _AppClient_decryptionCache, "f").get(randomId) || new Map();
    }
    /**
     * Close current event subscription.
     */
    async close() { }
    /**
     * Exit current game.
     */
    async exit() {
        await __classPrivateFieldGet(this, _AppClient_connection, "f").exitGame(__classPrivateFieldGet(this, _AppClient_gameAddr, "f"), {});
    }
    get info() {
        return __classPrivateFieldGet(this, _AppClient_info, "f");
    }
}
_AppClient_gameAddr = new WeakMap(), _AppClient_handler = new WeakMap(), _AppClient_wallet = new WeakMap(), _AppClient_client = new WeakMap(), _AppClient_transport = new WeakMap(), _AppClient_connection = new WeakMap(), _AppClient_gameContext = new WeakMap(), _AppClient_initGameAccount = new WeakMap(), _AppClient_onEvent = new WeakMap(), _AppClient_onMessage = new WeakMap(), _AppClient_encryptor = new WeakMap(), _AppClient_profileCaches = new WeakMap(), _AppClient_info = new WeakMap(), _AppClient_decryptionCache = new WeakMap();
//# sourceMappingURL=app-client.js.map